@@ -267,7 +267,7 @@ def _get_host_by_name(host):
     ip_address : string
         resolved IP address
     """
-    dns_url = f"https://dns.google/resolve?name={host}"
+    dns_url = f"https://8.8.8.8/resolve?name={host}"
     response = requests.get(dns_url)
     data = response.json()
 
