@@ -1,6 +1,7 @@
 """Graph creation functions."""
 
 import itertools
+import warnings
 
 import networkx as nx
 from shapely.geometry import MultiPolygon
@@ -13,6 +14,7 @@ from . import osm_xml
 from . import projection
 from . import settings
 from . import simplification
+from . import stats
 from . import truncate
 from . import utils
 from . import utils_geo
@@ -72,7 +74,9 @@ def graph_from_bbox(
     Notes
     -----
     You can configure the Overpass server timeout, memory allocation, and
-    other custom settings via ox.config().
+    other custom settings via the `settings` module. Very large query areas
+    will use the utils_geo._consolidate_subdivide_geometry function to perform
+    multiple queries: see that function's documentation for caveats.
     """
     # convert bounding box to a polygon
     polygon = utils_geo.bbox_to_poly(north, south, east, west)
@@ -114,9 +118,10 @@ def graph_from_point(
         retain only those nodes within this many meters of the center of the
         graph, with distance determined according to dist_type argument
     dist_type : string {"network", "bbox"}
-        if "bbox", retain only those nodes within a bounding box of the distance
-        parameter. if "network", retain only those nodes within some network
-        distance from the center-most node.
+        if "bbox", retain only those nodes within a bounding box of the
+        distance parameter. if "network", retain only those nodes within some
+        network distance from the center-most node (requires that scikit-learn
+        is installed as an optional dependency).
     network_type : string, {"all_private", "all", "bike", "drive", "drive_service", "walk"}
         what type of street network to get if custom_filter is None
     simplify : bool
@@ -143,13 +148,14 @@ def graph_from_point(
     Notes
     -----
     You can configure the Overpass server timeout, memory allocation, and
-    other custom settings via ox.config().
+    other custom settings via the `settings` module. Very large query areas
+    will use the utils_geo._consolidate_subdivide_geometry function to perform
+    multiple queries: see that function's documentation for caveats.
     """
-    if dist_type not in {"bbox", "network"}:
+    if dist_type not in {"bbox", "network"}:  # pragma: no cover
         raise ValueError('dist_type must be "bbox" or "network"')
 
-    # create a bounding box from the center point and the distance in each
-    # direction
+    # create bounding box from center point and distance in each direction
     north, south, east, west = utils_geo.bbox_from_point(center_point, dist)
 
     # create a graph from the bounding box
@@ -166,11 +172,11 @@ def graph_from_point(
         custom_filter=custom_filter,
     )
 
-    # if network dist_type is network, find the node in the graph nearest to
-    # the center point, and truncate the graph by network distance from node
     if dist_type == "network":
-        centermost_node = distance.get_nearest_node(G, center_point)
-        G = truncate.truncate_graph_dist(G, centermost_node, max_dist=dist)
+        # if dist_type is network, find node in graph nearest to center point
+        # then truncate graph by network dist from it
+        node = distance.nearest_nodes(G, X=[center_point[1]], Y=[center_point[0]])[0]
+        G = truncate.truncate_graph_dist(G, node, max_dist=dist)
 
     utils.log(f"graph_from_point returned graph with {len(G)} nodes and {len(G.edges)} edges")
     return G
@@ -200,9 +206,10 @@ def graph_from_address(
         retain only those nodes within this many meters of the center of the
         graph
     dist_type : string {"network", "bbox"}
-        if "bbox", retain only those nodes within a bounding box of the distance
-        parameter. if "network", retain only those nodes within some network
-        distance from the center-most node.
+        if "bbox", retain only those nodes within a bounding box of the
+        distance parameter. if "network", retain only those nodes within some
+        network distance from the center-most node (requires that scikit-learn
+        is installed as an optional dependency).
     network_type : string {"all_private", "all", "bike", "drive", "drive_service", "walk"}
         what type of street network to get if custom_filter is None
     simplify : bool
@@ -231,7 +238,9 @@ def graph_from_address(
     Notes
     -----
     You can configure the Overpass server timeout, memory allocation, and
-    other custom settings via ox.config().
+    other custom settings via the `settings` module. Very large query areas
+    will use the utils_geo._consolidate_subdivide_geometry function to perform
+    multiple queries: see that function's documentation for caveats.
     """
     # geocode the address string to a (lat, lng) point
     point = geocoder.geocode(query=address)
@@ -317,7 +326,9 @@ def graph_from_place(
     Notes
     -----
     You can configure the Overpass server timeout, memory allocation, and
-    other custom settings via ox.config().
+    other custom settings via the `settings` module. Very large query areas
+    will use the utils_geo._consolidate_subdivide_geometry function to perform
+    multiple queries: see that function's documentation for caveats.
     """
     # create a GeoDataFrame with the spatial boundaries of the place(s)
     if isinstance(query, (str, dict)):
@@ -329,7 +340,7 @@ def graph_from_place(
     elif isinstance(query, list):
         # if it is a list, it contains multiple places to get
         gdf_place = geocoder.geocode_to_gdf(query, buffer_dist=buffer_dist)
-    else:
+    else:  # pragma: no cover
         raise TypeError("query must be dict, string, or list of strings")
 
     # extract the geometry from the GeoDataFrame to use in API query
@@ -394,13 +405,15 @@ def graph_from_polygon(
     Notes
     -----
     You can configure the Overpass server timeout, memory allocation, and
-    other custom settings via ox.config().
+    other custom settings via the `settings` module. Very large query areas
+    will use the utils_geo._consolidate_subdivide_geometry function to perform
+    multiple queries: see that function's documentation for caveats.
     """
     # verify that the geometry is valid and is a shapely Polygon/MultiPolygon
     # before proceeding
-    if not polygon.is_valid:
+    if not polygon.is_valid:  # pragma: no cover
         raise ValueError("The geometry to query within is invalid")
-    if not isinstance(polygon, (Polygon, MultiPolygon)):
+    if not isinstance(polygon, (Polygon, MultiPolygon)):  # pragma: no cover
         raise TypeError(
             "Geometry must be a shapely Polygon or MultiPolygon. If you requested "
             "graph from place name, make sure your query resolves to a Polygon or "
@@ -442,7 +455,7 @@ def graph_from_polygon(
         # count how many physical streets in buffered graph connect to each
         # intersection in un-buffered graph, to retain true counts for each
         # intersection, even if some of its neighbors are outside the polygon
-        spn = utils_graph.count_streets_per_node(G_buff, nodes=G.nodes)
+        spn = stats.count_streets_per_node(G_buff, nodes=G.nodes)
         nx.set_node_attributes(G, values=spn, name="street_count")
 
     # if clean_periphery=False, just use the polygon as provided
@@ -457,12 +470,23 @@ def graph_from_polygon(
         # truncate the graph to the extent of the polygon
         G = truncate.truncate_graph_polygon(G, polygon, retain_all, truncate_by_edge)
 
-        # simplify the graph topology as the last step. don't truncate after
+        # simplify the graph topology after truncation. don't truncate after
         # simplifying or you may have simplified out to an endpoint beyond the
         # truncation distance, which would strip out the entire edge
         if simplify:
             G = simplification.simplify_graph(G)
 
+        # count how many physical streets connect to each intersection/deadend
+        # note this will be somewhat inaccurate due to periphery effects, so
+        # it's best to parameterize function with clean_periphery=True
+        spn = stats.count_streets_per_node(G)
+        nx.set_node_attributes(G, values=spn, name="street_count")
+        msg = (
+            "the graph-level street_count attribute will likely be inaccurate "
+            "when you set clean_periphery=False"
+        )
+        warnings.warn(msg)
+
     utils.log(f"graph_from_polygon returned graph with {len(G)} nodes and {len(G.edges)} edges")
     return G
 
@@ -507,7 +531,7 @@ def _create_graph(response_jsons, retain_all=False, bidirectional=False):
 
     Adds length attributes in meters (great-circle distance between endpoints)
     to all of the graph's (pre-simplified, straight-line) edges via the
-    `utils_graph.add_edge_lengths` function.
+    `distance.add_edge_lengths` function.
 
     Parameters
     ----------
@@ -526,7 +550,7 @@ def _create_graph(response_jsons, retain_all=False, bidirectional=False):
     utils.log("Creating graph from downloaded OSM data...")
 
     # make sure we got data back from the server request(s)
-    if not any(rj["elements"] for rj in response_jsons):
+    if not any(rj["elements"] for rj in response_jsons):  # pragma: no cover
         raise EmptyOverpassResponse("There are no data elements in the response JSON")
 
     # create the graph as a MultiDiGraph and set its meta-attributes
@@ -560,7 +584,7 @@ def _create_graph(response_jsons, retain_all=False, bidirectional=False):
 
     # add length (great-circle distance between nodes) attribute to each edge
     if len(G.edges) > 0:
-        G = utils_graph.add_edge_lengths(G)
+        G = distance.add_edge_lengths(G)
 
     return G
 
@@ -748,12 +772,15 @@ def _add_paths(G, paths, bidirectional=False):
         if not settings.all_oneway:
             path["oneway"] = is_one_way
 
-        # zip path nodes to get (u, v) tuples like [(0,1), (1,2), (2,3)]. if
-        # the path is NOT one-way, reverse direction of each edge and add this
-        # path going the opposite direction too
+        # zip path nodes to get (u, v) tuples like [(0,1), (1,2), (2,3)].
         edges = list(zip(nodes[:-1], nodes[1:]))
-        if not is_one_way:
-            edges.extend([(v, u) for u, v in edges])
 
         # add all the edge tuples and give them the path's tag:value attrs
+        path["reversed"] = False
         G.add_edges_from(edges, **path)
+
+        # if the path is NOT one-way, reverse direction of each edge and add
+        # this path going the opposite direction too
+        if not is_one_way:
+            path["reversed"] = True
+            G.add_edges_from([(v, u) for u, v in edges], **path)
