@@ -5,6 +5,7 @@ import logging as lg
 import geopandas as gpd
 import networkx as nx
 from shapely.geometry import LineString
+from shapely.geometry import MultiPolygon
 from shapely.geometry import Point
 from shapely.geometry import Polygon
 
@@ -226,6 +227,9 @@ def simplify_graph(G, strict=True, remove_rings=True):
 
     utils.log("Begin topologically simplifying the graph...")
 
+    # define edge segment attributes to sum upon edge simplification
+    attrs_to_sum = {"length", "travel_time"}
+
     # make a copy to not mutate original graph object caller passed in
     G = G.copy()
     initial_node_count = len(G)
@@ -238,48 +242,51 @@ def simplify_graph(G, strict=True, remove_rings=True):
 
         # add the interstitial edges we're removing to a list so we can retain
         # their spatial geometry
-        edge_attributes = dict()
+        path_attributes = dict()
         for u, v in zip(path[:-1], path[1:]):
 
             # there should rarely be multiple edges between interstitial nodes
             # usually happens if OSM has duplicate ways digitized for just one
             # street... we will keep only one of the edges (see below)
-            if G.number_of_edges(u, v) != 1:
-                utils.log(f"Found multiple edges between {u} and {v} when simplifying")
+            edge_count = G.number_of_edges(u, v)
+            if edge_count != 1:
+                utils.log(f"Found {edge_count} edges between {u} and {v} when simplifying")
 
             # get edge between these nodes: if multiple edges exist between
             # them (see above), we retain only one in the simplified graph
-            edge = G.edges[u, v, 0]
-            for key in edge:
-                if key in edge_attributes:
+            edge_data = G.edges[u, v, 0]
+            for attr in edge_data:
+                if attr in path_attributes:
                     # if this key already exists in the dict, append it to the
                     # value list
-                    edge_attributes[key].append(edge[key])
+                    path_attributes[attr].append(edge_data[attr])
                 else:
                     # if this key doesn't already exist, set the value to a list
                     # containing the one value
-                    edge_attributes[key] = [edge[key]]
+                    path_attributes[attr] = [edge_data[attr]]
 
-        for key in edge_attributes:
-            # don't touch the length attribute, we'll sum it at the end
-            if len(set(edge_attributes[key])) == 1 and key != "length":
+        # consolidate the path's edge segments' attribute values
+        for attr in path_attributes:
+            if attr in attrs_to_sum:
+                # if this attribute must be summed, sum it now
+                path_attributes[attr] = sum(path_attributes[attr])
+            elif len(set(path_attributes[attr])) == 1:
                 # if there's only 1 unique value in this attribute list,
-                # consolidate it to the single value (the zero-th)
-                edge_attributes[key] = edge_attributes[key][0]
-            elif key != "length":
-                # otherwise, if there are multiple values, keep one of each value
-                edge_attributes[key] = list(set(edge_attributes[key]))
+                # consolidate it to the single value (the zero-th):
+                path_attributes[attr] = path_attributes[attr][0]
+            else:
+                # otherwise, if there are multiple values, keep one of each
+                path_attributes[attr] = list(set(path_attributes[attr]))
 
-        # construct the geometry and sum the lengths of the segments
-        edge_attributes["geometry"] = LineString(
+        # construct the new consolidated edge's geometry for this path
+        path_attributes["geometry"] = LineString(
             [Point((G.nodes[node]["x"], G.nodes[node]["y"])) for node in path]
         )
-        edge_attributes["length"] = sum(edge_attributes["length"])
 
-        # add the nodes and edges to their lists for processing at the end
+        # add the nodes and edge to their lists for processing at the end
         all_nodes_to_remove.extend(path[1:-1])
         all_edges_to_add.append(
-            {"origin": path[0], "destination": path[-1], "attr_dict": edge_attributes}
+            {"origin": path[0], "destination": path[-1], "attr_dict": path_attributes}
         )
 
     # for each edge to add in the list we assembled, create a new edge between
@@ -322,6 +329,8 @@ def consolidate_intersections(
     tolerance argument should be adjusted to approximately match street design
     standards in the specific street network, and you should always use a
     projected graph to work in meaningful and consistent units like meters.
+    Note the tolerance represents a per-node buffering radius: for example, to
+    consolidate nodes within 10 meters of each other, use tolerance=5.
 
     When rebuild_graph=False, it uses a purely geometrical (and relatively
     fast) algorithm to identify "geometrically close" nodes, merge them, and
@@ -417,10 +426,9 @@ def _merge_nodes_geometric(G, tolerance):
     merged = utils_graph.graph_to_gdfs(G, edges=False)["geometry"].buffer(tolerance).unary_union
 
     # if only a single node results, make it iterable to convert to GeoSeries
-    if isinstance(merged, Polygon):
-        merged = [merged]
+    merged = MultiPolygon([merged]) if isinstance(merged, Polygon) else merged
 
-    return gpd.GeoSeries(list(merged), crs=G.graph["crs"])
+    return gpd.GeoSeries(merged.geoms, crs=G.graph["crs"])
 
 
 def _consolidate_intersections_rebuild_graph(G, tolerance=10, reconnect_edges=True):
@@ -472,7 +480,7 @@ def _consolidate_intersections_rebuild_graph(G, tolerance=10, reconnect_edges=Tr
     # graph's node points then spatial join to give each node the label of
     # cluster it's within
     node_points = utils_graph.graph_to_gdfs(G, edges=False)[["geometry"]]
-    gdf = gpd.sjoin(node_points, node_clusters, how="left", op="within")
+    gdf = gpd.sjoin(node_points, node_clusters, how="left", predicate="within")
     gdf = gdf.drop(columns="geometry").rename(columns={"index_right": "cluster"})
 
     # STEP 3
@@ -490,11 +498,12 @@ def _consolidate_intersections_rebuild_graph(G, tolerance=10, reconnect_edges=Tr
                 suffix = 0
                 for wcc in wccs:
                     # set subcluster xy to the centroid of just these nodes
-                    subcluster_centroid = node_points.loc[wcc].unary_union.centroid
-                    gdf.loc[wcc, "x"] = subcluster_centroid.x
-                    gdf.loc[wcc, "y"] = subcluster_centroid.y
+                    idx = list(wcc)
+                    subcluster_centroid = node_points.loc[idx].unary_union.centroid
+                    gdf.loc[idx, "x"] = subcluster_centroid.x
+                    gdf.loc[idx, "y"] = subcluster_centroid.y
                     # move to subcluster by appending suffix to cluster label
-                    gdf.loc[wcc, "cluster"] = f"{cluster_label}-{suffix}"
+                    gdf.loc[idx, "cluster"] = f"{cluster_label}-{suffix}"
                     suffix += 1
 
     # give nodes unique integer IDs (subclusters with suffixes are strings)
@@ -526,6 +535,11 @@ def _consolidate_intersections_rebuild_graph(G, tolerance=10, reconnect_edges=Tr
                 y=nodes_subset["y"].iloc[0],
             )
 
+    # calculate street_count attribute for all nodes lacking it
+    null_nodes = [n for n, sc in H.nodes(data="street_count") if sc is None]
+    street_count = stats.count_streets_per_node(H, nodes=null_nodes)
+    nx.set_node_attributes(H, street_count, name="street_count")
+
     if not G.edges or not reconnect_edges:
         # if reconnect_edges is False or there are no edges in original graph
         # (after dead-end removed), then skip edges and return new graph as-is
