@@ -2,11 +2,13 @@
 
 import datetime as dt
 import logging as lg
+import os
 import sys
 import unicodedata
+import warnings
+from contextlib import redirect_stdout
 from pathlib import Path
 
-from . import _version
 from . import settings
 
 
@@ -87,7 +89,6 @@ def config(
     default_crs=settings.default_crs,
     default_referer=settings.default_referer,
     default_user_agent=settings.default_user_agent,
-    elevation_provider=settings.elevation_provider,
     imgs_folder=settings.imgs_folder,
     log_console=settings.log_console,
     log_file=settings.log_file,
@@ -106,109 +107,95 @@ def config(
     overpass_endpoint=settings.overpass_endpoint,
     overpass_rate_limit=settings.overpass_rate_limit,
     overpass_settings=settings.overpass_settings,
+    requests_kwargs=settings.requests_kwargs,
     timeout=settings.timeout,
     use_cache=settings.use_cache,
     useful_tags_node=settings.useful_tags_node,
     useful_tags_way=settings.useful_tags_way,
 ):
     """
-    Configure OSMnx by setting the default global settings' values.
-
-    Any parameters not passed by the caller are (re-)set to their original
-    default values.
+    Do not use: deprecated. Use the settings module directly.
 
     Parameters
     ----------
     all_oneway : bool
-        Only use if specifically saving to .osm XML file with save_graph_xml
-        function. if True, forces all ways to be loaded as oneway ways,
-        preserving the original order of nodes stored in the OSM way XML.
-        This also retains original OSM string values for oneway attribute
-        values, rather than converting them to a True/False bool.
+        deprecated
     bidirectional_network_types : list
-        network types for which a fully bidirectional graph will be created
+        deprecated
     cache_folder : string or pathlib.Path
-        path to folder in which to save/load HTTP response cache
+        deprecated
     data_folder : string or pathlib.Path
-        path to folder in which to save/load graph files by default
+        deprecated
     cache_only_mode : bool
-        If True, download network data from Overpass then raise a
-        CacheOnlyModeInterrupt error for user to catch. This prevents graph
-        building from taking place and instead just saves OSM response data to
-        cache. Useful for sequentially caching lots of raw data (as you can
-        only query Overpass one request at a time) then using the cache to
-        quickly build many graphs simultaneously with multiprocessing.
+        deprecated
     default_accept_language : string
-        HTTP header accept-language
+        deprecated
     default_access : string
-        default filter for OSM "access" key
+        deprecated
     default_crs : string
-        default coordinate reference system to set when creating graphs
+        deprecated
     default_referer : string
-        HTTP header referer
+        deprecated
     default_user_agent : string
-        HTTP header user-agent
-    elevation_provider : string {"google", "airmap"}
-        the API provider to use for adding node elevations
+        deprecated
     imgs_folder : string or pathlib.Path
-        path to folder in which to save plot images by default
+        deprecated
     log_file : bool
-        if True, save log output to a file in logs_folder
+        deprecated
     log_filename : string
-        name of the log file, without file extension
+        deprecated
     log_console : bool
-        if True, print log output to the console (terminal window)
+        deprecated
     log_level : int
-        one of Python's logger.level constants
+        deprecated
     log_name : string
-        name of the logger
+        deprecated
     logs_folder : string or pathlib.Path
-        path to folder in which to save log files
+        deprecated
     max_query_area_size : int
-        maximum area for any part of the geometry in meters: any polygon
-        bigger than this will get divided up for multiple queries to API
-        (default 50km x 50km)
+        deprecated
     memory : int
-        Overpass server memory allocation size for the query, in bytes. If
-        None, server will use its default allocation size. Use with caution.
+        deprecated
     nominatim_endpoint : string
-        base API endpoint to use for nominatim queries
+        deprecated
     nominatim_key : string
-        your API key, if you are using an endpoint that requires one
+        deprecated
     osm_xml_node_attrs : list
-        node attributes for saving .osm XML files with save_graph_xml function
+        deprecated
     osm_xml_node_tags : list
-        node tags for saving .osm XML files with save_graph_xml function
+        deprecated
     osm_xml_way_attrs : list
-        edge attributes for saving .osm XML files with save_graph_xml function
+        deprecated
     osm_xml_way_tags : list
-        edge tags for for saving .osm XML files with save_graph_xml function
+        deprecated
     overpass_endpoint : string
-        base API endpoint to use for overpass queries
+        deprecated
     overpass_rate_limit : bool
-        if True, check the overpass server status endpoint for how long to
-        pause before making request. Necessary if server uses slot management,
-        but can be set to False if you are running your own overpass instance.
+        deprecated
     overpass_settings : string
-        Settings string for overpass queries. For example, to query historical
-        OSM data as of a certain date:
-        ``'[out:json][timeout:90][date:"2019-10-28T19:20:00Z"]'``.
-        Use with caution.
+        deprecated
+    requests_kwargs : dict
+        deprecated
     timeout : int
-        the timeout interval for the HTTP request and for API to use while
-        running the query
+        deprecated
     use_cache : bool
-        if True, cache HTTP responses locally instead of calling API
-        repeatedly for the same request
+        deprecated
     useful_tags_node : list
-        OSM "node" tags to add as graph node attributes, when present
+        deprecated
     useful_tags_way : list
-        OSM "way" tags to add as graph edge attributes, when present
+        deprecated
 
     Returns
     -------
     None
     """
+    warnings.warn(
+        "The `utils.config` function is deprecated and will be removed in a "
+        "future release. Instead, use the `settings` module directly to "
+        "configure a global setting's value. For example, "
+        "`ox.settings.log_console=True`."
+    )
+
     # set each global setting to the argument value
     settings.all_oneway = all_oneway
     settings.bidirectional_network_types = bidirectional_network_types
@@ -220,7 +207,6 @@ def config(
     settings.default_crs = default_crs
     settings.default_referer = default_referer
     settings.default_user_agent = default_user_agent
-    settings.elevation_provider = elevation_provider
     settings.imgs_folder = imgs_folder
     settings.log_console = log_console
     settings.log_file = log_file
@@ -243,9 +229,7 @@ def config(
     settings.use_cache = use_cache
     settings.useful_tags_node = useful_tags_node
     settings.useful_tags_way = useful_tags_way
-
-    log(f"Configured OSMnx {_version.__version__}")
-    log(f"HTTP response caching is {'on' if settings.use_cache else 'off'}")
+    settings.requests_kwargs = requests_kwargs
 
 
 def log(message, level=None, name=None, filename=None):
@@ -291,14 +275,8 @@ def log(message, level=None, name=None, filename=None):
         elif level == lg.ERROR:
             logger.error(message)
 
-    # if logging to console, convert message to ascii and print to console
+    # if logging to console (terminal window) is turned on
     if settings.log_console:
-        # capture current stdout, then switch it to the console, print the
-        # message, then switch back to what had been the stdout. this prevents
-        # logging to notebook in jupyter: instead, it goes to terminal
-        standard_out = sys.stdout
-        sys.stdout = sys.__stdout__
-
         # prepend timestamp
         message = f"{ts()} {message}"
 
@@ -306,8 +284,14 @@ def log(message, level=None, name=None, filename=None):
         message = (
             unicodedata.normalize("NFKD", str(message)).encode("ascii", errors="replace").decode()
         )
-        print(message)
-        sys.stdout = standard_out
+
+        # print explicitly to terminal in case jupyter notebook is the stdout
+        if getattr(sys.stdout, "_original_stdstream_copy", None) is not None:
+            # redirect captured pipe back to original
+            os.dup2(sys.stdout._original_stdstream_copy, sys.__stdout__.fileno())
+            sys.stdout._original_stdstream_copy = None
+        with redirect_stdout(sys.__stdout__):
+            print(message, file=sys.__stdout__, flush=True)
 
 
 def _get_logger(level, name, filename):
