@@ -1,7 +1,8 @@
 """Geospatial utility functions."""
 
-import math
+import warnings
 
+import networkx as nx
 import numpy as np
 from shapely.geometry import LineString
 from shapely.geometry import MultiLineString
@@ -14,41 +15,68 @@ from shapely.ops import split
 from . import projection
 from . import settings
 from . import utils
+from . import utils_graph
 
 
-def redistribute_vertices(geom, dist):
+def sample_points(G, n):
     """
-    Redistribute the vertices on a projected LineString or MultiLineString.
+    Randomly sample points constrained to a spatial graph.
 
-    The distance argument is only approximate since the total distance of the
-    linestring may not be a multiple of the preferred distance. This function
-    works on only (Multi)LineString geometry types.
+    This generates a graph-constrained uniform random sample of points. Unlike
+    typical spatially uniform random sampling, this method accounts for the
+    graph's geometry. And unlike equal-length edge segmenting, this method
+    guarantees uniform randomness.
 
     Parameters
     ----------
-    geom : shapely.geometry.LineString or shapely.geometry.MultiLineString
-        a Shapely geometry (should be projected)
-    dist : float
-        spacing length along edges. Units are same as the geom: degrees for
-        unprojected geometries and meters for projected geometries. The
-        smaller the dist value, the more points are created.
+    G : networkx.MultiGraph
+        graph to sample points from; should be undirected (to not oversample
+        bidirectional edges) and projected (for accurate point interpolation)
+    n : int
+        how many points to sample
 
     Returns
     -------
-    list or shapely.geometry.MultiLineString
-        the redistributed vertices as a list if geom is a LineString or
-        MultiLineString if geom is a MultiLineString
+    points : geopandas.GeoSeries
+        the sampled points, multi-indexed by (u, v, key) of the edge from
+        which each point was drawn
     """
-    if geom.geom_type == "LineString":
-        num_vert = round(geom.length / dist)
-        if num_vert == 0:
-            num_vert = 1
-        return [geom.interpolate(n / num_vert, normalized=True) for n in range(num_vert + 1)]
-    elif geom.geom_type == "MultiLineString":
-        parts = [redistribute_vertices(part, dist) for part in geom]
-        return type(geom)([p for p in parts if not p])
-    else:
-        raise ValueError(f"unhandled geometry {geom.geom_type}")
+    if nx.is_directed(G):  # pragma: no cover
+        warnings.warn("graph should be undirected to not oversample bidirectional edges")
+    gdf_edges = utils_graph.graph_to_gdfs(G, nodes=False)[["geometry", "length"]]
+    weights = gdf_edges["length"] / gdf_edges["length"].sum()
+    idx = np.random.choice(gdf_edges.index, size=n, p=weights)
+    lines = gdf_edges.loc[idx, "geometry"]
+    return lines.interpolate(np.random.rand(n), normalized=True)
+
+
+def interpolate_points(geom, dist):
+    """
+    Interpolate evenly spaced points along a LineString.
+
+    The spacing is approximate because the LineString's length may not be
+    evenly divisible by it.
+
+    Parameters
+    ----------
+    geom : shapely.geometry.LineString
+        a LineString geometry
+    dist : float
+        spacing distance between interpolated points, in same units as `geom`.
+        smaller values generate more points.
+
+    Yields
+    ------
+    points : generator
+        a generator of (x, y) tuples of the interpolated points' coordinates
+    """
+    if isinstance(geom, LineString):
+        num_vert = max(round(geom.length / dist), 1)
+        for n in range(num_vert + 1):
+            point = geom.interpolate(n / num_vert, normalized=True)
+            yield point.x, point.y
+    else:  # pragma: no cover
+        raise TypeError(f"unhandled geometry type {geom.geom_type}")
 
 
 def _round_polygon_coords(p, precision):
@@ -64,21 +92,17 @@ def _round_polygon_coords(p, precision):
 
     Returns
     -------
-    new_poly : shapely.geometry.Polygon
-        the polygon with rounded coordinates
+    shapely.geometry.Polygon
     """
-    # round the coordinates of the Polygon exterior
-    new_exterior = [[round(x, precision) for x in c] for c in p.exterior.coords]
+    # round coords of Polygon exterior
+    shell = [[round(x, precision) for x in c] for c in p.exterior.coords]
 
-    # round the coordinates of the (possibly multiple, possibly none) Polygon interior(s)
-    new_interiors = []
-    for interior in p.interiors:
-        new_interiors.append([[round(x, precision) for x in c] for c in interior.coords])
+    # round coords of (possibly multiple, possibly none) Polygon interior(s)
+    holes = [[[round(x, precision) for x in c] for c in i.coords] for i in p.interiors]
 
-    # construct a new Polygon with the rounded coordinates
-    # buffer by zero to clean self-touching or self-crossing polygons
-    new_poly = Polygon(shell=new_exterior, holes=new_interiors).buffer(0)
-    return new_poly
+    # construct new Polygon with rounded coordinates and buffer by zero to
+    # clean self-touching or self-crossing polygons
+    return Polygon(shell=shell, holes=holes).buffer(0)
 
 
 def _round_multipolygon_coords(mp, precision):
@@ -96,7 +120,7 @@ def _round_multipolygon_coords(mp, precision):
     -------
     shapely.geometry.MultiPolygon
     """
-    return MultiPolygon([_round_polygon_coords(p, precision) for p in mp])
+    return MultiPolygon([_round_polygon_coords(p, precision) for p in mp.geoms])
 
 
 def _round_point_coords(pt, precision):
@@ -132,7 +156,7 @@ def _round_multipoint_coords(mpt, precision):
     -------
     shapely.geometry.MultiPoint
     """
-    return MultiPoint([_round_point_coords(pt, precision) for pt in mpt])
+    return MultiPoint([_round_point_coords(pt, precision) for pt in mpt.geoms])
 
 
 def _round_linestring_coords(ls, precision):
@@ -168,7 +192,7 @@ def _round_multilinestring_coords(mls, precision):
     -------
     shapely.geometry.MultiLineString
     """
-    return MultiLineString([_round_linestring_coords(ls, precision) for ls in mls])
+    return MultiLineString([_round_linestring_coords(ls, precision) for ls in mls.geoms])
 
 
 def round_geometry_coords(geom, precision):
@@ -204,7 +228,7 @@ def round_geometry_coords(geom, precision):
     elif isinstance(geom, MultiPolygon):
         return _round_multipolygon_coords(geom, precision)
 
-    else:
+    else:  # pragma: no cover
         raise TypeError(f"cannot round coordinates of unhandled geometry type: {type(geom)}")
 
 
@@ -216,6 +240,11 @@ def _consolidate_subdivide_geometry(geometry, max_query_area_size=None):
     sub-polygons if its area exceeds max size (in geometry's units). Configure
     the max size via max_query_area_size in the settings module.
 
+    When the geometry has a very large area relative to its vertex count,
+    the resulting MultiPolygon's boundary may differ somewhat from the input,
+    due to the way long straight lines are projected. You can interpolate
+    additional vertices along your input geometry's exterior to mitigate this.
+
     Parameters
     ----------
     geometry : shapely.geometry.Polygon or shapely.geometry.MultiPolygon
@@ -227,20 +256,20 @@ def _consolidate_subdivide_geometry(geometry, max_query_area_size=None):
 
     Returns
     -------
-    geometry : shapely.geometry.Polygon or shapely.geometry.MultiPolygon
+    geometry : shapely.geometry.MultiPolygon
     """
     if max_query_area_size is None:
         max_query_area_size = settings.max_query_area_size
 
     # let the linear length of the quadrats (with which to subdivide the
     # geometry) be the square root of max area size
-    quadrat_width = math.sqrt(max_query_area_size)
+    quadrat_width = np.sqrt(max_query_area_size)
 
-    if not isinstance(geometry, (Polygon, MultiPolygon)):
+    if not isinstance(geometry, (Polygon, MultiPolygon)):  # pragma: no cover
         raise TypeError("Geometry must be a shapely Polygon or MultiPolygon")
 
-    # if geometry is a MultiPolygon OR a single Polygon whose area exceeds the
-    # max size, get the convex hull around the geometry
+    # if geometry is either 1) a Polygon whose area exceeds the max size, or
+    # 2) a MultiPolygon, then get the convex hull around the geometry
     if isinstance(geometry, MultiPolygon) or (
         isinstance(geometry, Polygon) and geometry.area > max_query_area_size
     ):
@@ -271,26 +300,22 @@ def _get_polygons_coordinates(geometry):
     -------
     polygon_coord_strs : list
     """
-    # extract the exterior coordinates of the geometry to pass to the API later
-    polygons_coords = []
-    if isinstance(geometry, Polygon):
-        x, y = geometry.exterior.xy
-        polygons_coords.append(list(zip(x, y)))
-    elif isinstance(geometry, MultiPolygon):
-        for polygon in geometry:
-            x, y = polygon.exterior.xy
-            polygons_coords.append(list(zip(x, y)))
-    else:
-        raise TypeError("Geometry must be a shapely Polygon or MultiPolygon")
+    if not isinstance(geometry, MultiPolygon):  # pragma: no cover
+        raise TypeError("Geometry must be a shapely MultiPolygon")
 
-    # convert the exterior coordinates of the polygon(s) to the string format
-    # the API expects
+    # extract geometry's exterior coords
+    polygons_coords = []
+    for polygon in geometry.geoms:
+        x, y = polygon.exterior.xy
+        polygons_coords.append(list(zip(x, y)))
+
+    # convert exterior coords to the string format the API expects
     polygon_coord_strs = []
     for coords in polygons_coords:
         s = ""
         separator = " "
         for coord in list(coords):
-            # round floating point lats and longs to 6 decimal places (ie, ~100 mm),
+            # round floating point lats and longs to 6 decimals (ie, ~100 mm)
             # so we can hash and cache strings consistently
             s = f"{s}{separator}{coord[1]:.6f}{separator}{coord[0]:.6f}"
         polygon_coord_strs.append(s.strip(separator))
@@ -319,8 +344,8 @@ def _quadrat_cut_geometry(geometry, quadrat_width, min_num=3):
     """
     # create n evenly spaced points between the min and max x and y bounds
     west, south, east, north = geometry.bounds
-    x_num = math.ceil((east - west) / quadrat_width) + 1
-    y_num = math.ceil((north - south) / quadrat_width) + 1
+    x_num = int(np.ceil((east - west) / quadrat_width) + 1)
+    y_num = int(np.ceil((north - south) / quadrat_width) + 1)
     x_points = np.linspace(west, east, num=max(x_num, min_num))
     y_points = np.linspace(south, north, num=max(y_num, min_num))
 
@@ -330,10 +355,15 @@ def _quadrat_cut_geometry(geometry, quadrat_width, min_num=3):
     lines = vertical_lines + horizont_lines
 
     # recursively split the geometry by each quadrat line
-    for line in lines:
-        geometry = MultiPolygon(split(geometry, line))
+    geometries = [geometry]
 
-    return geometry
+    for line in lines:
+        # split polygon by line if they intersect, otherwise just keep it
+        split_geoms = [split(g, line).geoms if g.intersects(line) else [g] for g in geometries]
+        # now flatten the list and process these split geoms on the next line in the list of lines
+        geometries = [g for g_list in split_geoms for g in g_list]
+
+    return MultiPolygon(geometries)
 
 
 def _intersect_index_quadrats(geometries, polygon, quadrat_width=0.05, min_num=3):
@@ -371,7 +401,7 @@ def _intersect_index_quadrats(geometries, polygon, quadrat_width=0.05, min_num=3
     geoms_in_poly = set()
 
     # loop through each chunk of the polygon to find intersecting geometries
-    for poly in multipoly:
+    for poly in multipoly.geoms:
         # first find approximate matches with spatial index, then precise
         # matches from those approximate ones
         poly = poly.buffer(0)
@@ -411,8 +441,8 @@ def bbox_from_point(point, dist=1000, project_utm=False, return_crs=False):
     earth_radius = 6_371_009  # meters
     lat, lng = point
 
-    delta_lat = (dist / earth_radius) * (180 / math.pi)
-    delta_lng = (dist / earth_radius) * (180 / math.pi) / math.cos(lat * math.pi / 180)
+    delta_lat = (dist / earth_radius) * (180 / np.pi)
+    delta_lng = (dist / earth_radius) * (180 / np.pi) / np.cos(lat * np.pi / 180)
     north = lat + delta_lat
     south = lat - delta_lat
     east = lng + delta_lng
