@@ -1,7 +1,7 @@
 """Graph utility functions."""
 
 import itertools
-from collections import Counter
+import warnings
 
 import geopandas as gpd
 import networkx as nx
@@ -10,7 +10,6 @@ import pandas as pd
 from shapely.geometry import LineString
 from shapely.geometry import Point
 
-from . import distance
 from . import utils
 
 
@@ -29,19 +28,24 @@ def graph_to_gdfs(G, nodes=True, edges=True, node_geometry=True, fill_edge_geome
     edges : bool
         if True, convert graph edges to a GeoDataFrame and return it
     node_geometry : bool
-        if True, create a geometry column from node x and y data
+        if True, create a geometry column from node x and y attributes
     fill_edge_geometry : bool
         if True, fill in missing edge geometry fields using nodes u and v
 
     Returns
     -------
     geopandas.GeoDataFrame or tuple
-        gdf_nodes or gdf_edges or tuple of (gdf_nodes, gdf_edges)
+        gdf_nodes or gdf_edges or tuple of (gdf_nodes, gdf_edges). gdf_nodes
+        is indexed by osmid and gdf_edges is multi-indexed by u, v, key
+        following normal MultiDiGraph structure.
     """
     crs = G.graph["crs"]
 
     if nodes:
 
+        if not G.nodes:  # pragma: no cover
+            raise ValueError("graph contains no nodes")
+
         nodes, data = zip(*G.nodes(data=True))
 
         if node_geometry:
@@ -56,8 +60,8 @@ def graph_to_gdfs(G, nodes=True, edges=True, node_geometry=True, fill_edge_geome
 
     if edges:
 
-        if not G.edges:
-            raise ValueError("Graph has no edges, cannot convert to a GeoDataFrame.")
+        if not G.edges:  # pragma: no cover
+            raise ValueError("graph contains no edges")
 
         u, v, k, data = zip(*G.edges(keys=True, data=True))
 
@@ -99,8 +103,8 @@ def graph_to_gdfs(G, nodes=True, edges=True, node_geometry=True, fill_edge_geome
         return gdf_nodes
     elif edges:
         return gdf_edges
-    else:
-        raise ValueError("You must request nodes or edges or both.")
+    else:  # pragma: no cover
+        raise ValueError("you must request nodes or edges or both")
 
 
 def graph_from_gdfs(gdf_nodes, gdf_edges, graph_attrs=None):
@@ -108,12 +112,16 @@ def graph_from_gdfs(gdf_nodes, gdf_edges, graph_attrs=None):
     Convert node and edge GeoDataFrames to a MultiDiGraph.
 
     This function is the inverse of `graph_to_gdfs` and is designed to work in
-    conjunction with it. However, you can convert arbitrary node and edge
-    GeoDataFrames as long as gdf_nodes is uniquely indexed by `osmid` and
-    gdf_edges is uniquely multi-indexed by `u`, `v`, `key` (following normal
-    MultiDiGraph structure). This allows you to load any node/edge shapefiles
-    or GeoPackage layers as GeoDataFrames then convert them to a MultiDiGraph
-    for graph analysis.
+    conjunction with it.
+
+    However, you can convert arbitrary node and edge GeoDataFrames as long as
+    1) `gdf_nodes` is uniquely indexed by `osmid`, 2) `gdf_nodes` contains `x`
+    and `y` coordinate columns representing node geometries, 3) `gdf_edges` is
+    uniquely multi-indexed by `u`, `v`, `key` (following normal MultiDiGraph
+    structure). This allows you to load any node/edge shapefiles or GeoPackage
+    layers as GeoDataFrames then convert them to a MultiDiGraph for graph
+    analysis. Note that any `geometry` attribute on `gdf_nodes` is discarded
+    since `x` and `y` provide the necessary node geometry information instead.
 
     Parameters
     ----------
@@ -129,6 +137,27 @@ def graph_from_gdfs(gdf_nodes, gdf_edges, graph_attrs=None):
     -------
     G : networkx.MultiDiGraph
     """
+    if not ("x" in gdf_nodes.columns and "y" in gdf_nodes.columns):  # pragma: no cover
+        raise ValueError("gdf_nodes must contain x and y columns")
+
+    # if gdf_nodes has a geometry attribute set, drop that column (as we use x
+    # and y for geometry information) and warn the user if the geometry values
+    # differ from the coordinates in the x and y columns
+    if hasattr(gdf_nodes, "geometry"):
+        try:
+            all_x_match = (gdf_nodes.geometry.x == gdf_nodes["x"]).all()
+            all_y_match = (gdf_nodes.geometry.y == gdf_nodes["y"]).all()
+            assert all_x_match and all_y_match
+        except (AssertionError, ValueError):  # pragma: no cover
+            # AssertionError if x/y coords don't match geometry column
+            # ValueError if geometry column contains non-point geometry types
+            warnings.warn(
+                "discarding the gdf_nodes geometry column, though its "
+                "values differ from the coordinates in the x and y columns"
+            )
+        gdf_nodes = gdf_nodes.drop(columns=gdf_nodes.geometry.name)
+
+    # create graph and add graph-level attribute dict
     if graph_attrs is None:
         graph_attrs = {"crs": gdf_edges.crs}
     G = nx.MultiDiGraph(**graph_attrs)
@@ -141,7 +170,10 @@ def graph_from_gdfs(gdf_nodes, gdf_edges, graph_attrs=None):
         data = {name: val for name, val in data_all if isinstance(val, list) or pd.notnull(val)}
         G.add_edge(u, v, key=k, **data)
 
-    # add nodes' attributes to graph
+    # add any nodes with no incident edges, since they wouldn't be added above
+    G.add_nodes_from(set(gdf_nodes.index) - set(G.nodes))
+
+    # now all nodes are added, so set nodes' attributes
     for col in gdf_nodes.columns:
         nx.set_node_attributes(G, name=col, values=gdf_nodes[col].dropna())
 
@@ -149,101 +181,6 @@ def graph_from_gdfs(gdf_nodes, gdf_edges, graph_attrs=None):
     return G
 
 
-def add_edge_lengths(G, precision=3):
-    """
-    Add `length` attribute (in meters) to each edge.
-
-    Calculated via great-circle distance between each edge's incident nodes,
-    so ensure graph is in unprojected coordinates. Graph should be
-    unsimplified to get accurate distances. Note: this function is run by all
-    the `graph.graph_from_x` functions automatically to add `length`
-    attributes to all edges.
-
-    Parameters
-    ----------
-    G : networkx.MultiDiGraph
-        input graph
-    precision : int
-        decimal precision to round lengths
-
-    Returns
-    -------
-    G : networkx.MultiDiGraph
-        graph with edge length attributes
-    """
-    # extract the edges' endpoint nodes' coordinates
-    try:
-        coords = (
-            (u, v, k, G.nodes[u]["y"], G.nodes[u]["x"], G.nodes[v]["y"], G.nodes[v]["x"])
-            for u, v, k in G.edges
-        )
-    except KeyError:  # pragma: no cover
-        raise KeyError("Some edges missing nodes, possibly due to input data clipping issue")
-
-    # turn the coordinates into a DataFrame indexed by u, v, k
-    cols = ["u", "v", "k", "u_y", "u_x", "v_y", "v_x"]
-    df = pd.DataFrame(coords, columns=cols).set_index(["u", "v", "k"])
-
-    # calculate great circle distances, fill nulls with zeros, then round
-    dists = distance.great_circle_vec(df["u_y"], df["u_x"], df["v_y"], df["v_x"])
-    dists = dists.fillna(value=0).round(precision)
-    nx.set_edge_attributes(G, name="length", values=dists)
-
-    utils.log("Added edge lengths to graph")
-    return G
-
-
-def count_streets_per_node(G, nodes=None):
-    """
-    Count how many physical street segments connect to each node in a graph.
-
-    This function uses an undirected representation of the graph and special
-    handling of self-loops to accurately count physical streets rather than
-    directed edges. Note: this function is automatically run by all the
-    `graph.graph_from_x` functions prior to truncating the graph to the
-    requested boundaries, to add accurate `street_count` attributes to each
-    node even if some of its neighbors are outside the requested graph
-    boundaries.
-
-    Parameters
-    ----------
-    G : networkx.MultiDiGraph
-        input graph
-    nodes : list
-        which node IDs to get counts for. if None, use all graph nodes,
-        otherwise calculate counts only for these node IDs
-
-    Returns
-    -------
-    streets_per_node : dict
-        counts of how many physical streets connect to each node, with keys =
-        node ids and values = counts
-    """
-    if nodes is None:
-        nodes = G.nodes
-
-    # get one copy of each self-loop edge, because bi-directional self-loops
-    # appear twice in the undirected graph (u,v,0 and u,v,1 where u=v), but
-    # one-way self-loops will appear only once
-    Gu = G.to_undirected(reciprocal=False, as_view=True)
-    self_loop_edges = set(nx.selfloop_edges(Gu))
-
-    # get all non-self-loop undirected edges, including parallel edges
-    non_self_loop_edges = [e for e in Gu.edges(keys=False) if e not in self_loop_edges]
-
-    # make list of all unique edges including each parallel edge unless the
-    # parallel edge is a self-loop, in which case we don't double-count it
-    all_unique_edges = non_self_loop_edges + list(self_loop_edges)
-
-    # flatten list of (u, v) edge tuples to count how often each node appears
-    edges_flat = itertools.chain.from_iterable(all_unique_edges)
-    counts = Counter(edges_flat)
-    streets_per_node = {node: counts[node] for node in nodes}
-
-    utils.log("Counted undirected street segments incident to each node")
-    return streets_per_node
-
-
 def get_route_edge_attributes(
     G, route, attribute=None, minimize_key="length", retrieve_default=None
 ):
@@ -372,12 +309,13 @@ def get_digraph(G, weight="length"):
     to_remove = []
 
     # identify all the parallel edges in the MultiDiGraph
-    parallels = ((u, v) for u, v, k in G.edges(keys=True) if k > 0)
+    parallels = ((u, v) for u, v in G.edges(keys=False) if len(G.get_edge_data(u, v)) > 1)
 
-    # remove the parallel edge with greater "weight" attribute value
+    # among all sets of parallel edges, remove all except the one with the
+    # minimum "weight" attribute value
     for u, v in set(parallels):
-        k, _ = max(G.get_edge_data(u, v).items(), key=lambda x: x[1][weight])
-        to_remove.append((u, v, k))
+        k_min, _ = min(G.get_edge_data(u, v).items(), key=lambda x: x[1][weight])
+        to_remove.extend((u, v, k) for k in G[u][v] if k != k_min)
 
     G.remove_edges_from(to_remove)
     utils.log("Converted MultiDiGraph to DiGraph")
